# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('document_renaming', '0002_sequence_increment'),
    ]

    operations = [
        migrations.AlterField(
            model_name='sequence',
            name='slug',
            field=models.SlugField(
                help_text='Do not use hypens, if you need spacing use '
                'underscores.', verbose_name='Slug'
            ),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='sequence',
            name='value',
            field=models.IntegerField(default=0, verbose_name='Value'),
            preserve_default=True,
        ),
    ]



from __future__ import unicode_literals

import logging

from django.db import models
from django.template import Context, Template
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from documents.models import DocumentType

from .literals import (
    DEFAULT_SEQUENCE_INCREMENT, DEFAULT_SEQUENCE_INITIAL_VALUE
)
from .managers import RenamingTemplateManager

logger = logging.getLogger(__name__)


@python_2_unicode_compatible
class Sequence(models.Model):
    """
    Model to store the persistent state of a renaming sequence.
    """
    label = models.CharField(
        max_length=32, unique=True, verbose_name=_('Label')
    )
    slug = models.SlugField(
        help_text=_('Do not use hypens, if you need spacing use underscores.'),
        verbose_name=_('Slug')
    )
    increment = models.IntegerField(
        default=DEFAULT_SEQUENCE_INCREMENT, verbose_name=_('Increment')
    )
    value = models.IntegerField(
        default=DEFAULT_SEQUENCE_INITIAL_VALUE, verbose_name=_('Value')
    )

    def next_value(self):
        result = self.value
        self.value += self.increment
        self.save()
        return result

    class Meta:
        verbose_name = _('Sequence')
        verbose_name_plural = _('Sequences')

    def __str__(self):
        return self.label


class RenamingTemplate(models.Model):
    """
    Model to store the renaming template for a document type.
    """

    document_type = models.OneToOneField(
        DocumentType, related_name='renaming_template',
        verbose_name=_('Document type')
    )

    template = models.TextField(
        blank=True,
        help_text=_(
            'Sequences are available as "sequence_[sequence slug]". To obtain '
            'the next value for a sequence, call it\'s next_value method, ie: '
            '"invoice-{{ sequence_invoices.next_value }}.'
        ), verbose_name=_('Template')
    )

    objects = RenamingTemplateManager()

    def render_for(self, document):
        context_dictionary = {}
        for sequence in Sequence.objects.all():
            context_dictionary[
                'sequence_{}'.format(sequence.slug)
            ] = sequence
        context_dictionary['document'] = document

        try:
            template = Template(self.template)
            context = Context(context_dictionary)
            result = template.render(context=context)
        except Exception as exception:
            error_message = _(
                'Error rendering rename template for document: %(document)s; '
                'template: %(template)s; %(exception)s'
            ) % {
                'document': document,
                'template': self.template,
                'exception': exception
            }
            logger.debug(error_message)
        else:
            return result

    class Meta:
        verbose_name = _('Renaming template')
        verbose_name_plural = _('Renaming templates')



from __future__ import absolute_import, unicode_literals

from django.core.urlresolvers import reverse_lazy
from django.shortcuts import get_object_or_404
from django.utils.translation import ugettext_lazy as _

from common.generics import (
    SingleObjectCreateView, SingleObjectDeleteView, SingleObjectEditView,
    SingleObjectListView
)
from documents.models import DocumentType

from .models import Sequence
from .permissions import (
    permission_sequence_create, permission_sequence_delete,
    permission_sequence_edit, permission_sequence_view,
    permission_renaming_template_edit
)


class SequenceListView(SingleObjectListView):
    model = Sequence
    view_permission = permission_sequence_view

    def get_extra_context(self):
        return {
            'hide_object': True,
            'title': _('Sequences'),
        }


class SequenceCreateView(SingleObjectCreateView):
    model = Sequence
    post_action_redirect = reverse_lazy('document_renaming:sequence_list')
    view_permission = permission_sequence_create

    def get_extra_context(self):
        return {
            'title': _('Create sequence'),
        }


class SequenceDeleteView(SingleObjectDeleteView):
    model = Sequence
    post_action_redirect = reverse_lazy('document_renaming:sequence_list')
    view_permission = permission_sequence_delete

    def get_extra_context(self):
        return {
            'object': self.get_object(),
            'title': _('Delete the sequence: %s?') % self.get_object(),
        }


class SequenceEditView(SingleObjectEditView):
    model = Sequence
    post_action_redirect = reverse_lazy('document_renaming:sequence_list')
    view_permission = permission_sequence_edit

    def get_extra_context(self):
        return {
            'object': self.get_object(),
            'title': _('Edit sequence: %s') % self.get_object(),
        }


class DocumentTypeRenamingTemplateEditView(SingleObjectEditView):
    fields = ('template',)
    view_permission = permission_renaming_template_edit

    def get_object(self, queryset=None):
        return get_object_or_404(
            DocumentType, pk=self.kwargs['pk']
        ).renaming_template

    def get_extra_context(self):
        return {
            'title': _(
                'Edit renaming template for document type: %s'
            ) % self.get_object().document_type
        }



from __future__ import unicode_literals

from django.conf.urls import patterns, url

from .views import (
    SequenceListView, SequenceCreateView, SequenceDeleteView, SequenceEditView,
    DocumentTypeRenamingTemplateEditView
)

urlpatterns = patterns(
    '',
    url(
        r'^sequence/list/$', SequenceListView.as_view(),
        name='sequence_list'
    ),
    url(
        r'^sequence/create/$', SequenceCreateView.as_view(),
        name='sequence_create'
    ),
    url(
        r'^sequence/(?P<pk>\d+)/edit/$', SequenceEditView.as_view(),
        name='sequence_edit'
    ),
    url(
        r'^sequence/(?P<pk>\d+)/delete/$', SequenceDeleteView.as_view(),
        name='sequence_delete'
    ),
    url(
        r'^document_type/(?P<pk>\d+)/renaming_template/edit/$',
        DocumentTypeRenamingTemplateEditView.as_view(),
        name='renaming_template_edit'
    ),
)



from __future__ import unicode_literals

from django.core.files.base import File
from django.test import TestCase, override_settings

from documents.models import DocumentType, Document
from documents.tests import TEST_DOCUMENT_TYPE

from .literals import (
    TEST_SEQUENCE_LABEL, TEST_SEQUENCE_INCREMENT, TEST_SEQUENCE_INITIAL_VALUE,
    TEST_SEQUENCE_SLUG, TEST_SMALL_DOCUMENT_PATH, TEST_TEMPLATE
)

from ..models import Sequence


@override_settings(OCR_AUTO_OCR=False)
class SequenceTestCase(TestCase):
    def setUp(self):
        self.sequence = Sequence.objects.create(
            label=TEST_SEQUENCE_LABEL, slug=TEST_SEQUENCE_SLUG,
            increment=TEST_SEQUENCE_INCREMENT,
            value=TEST_SEQUENCE_INITIAL_VALUE
        )

    def tearDown(self):
        self.sequence.delete()

    def test_sequence_increment(self):
        self.assertEqual(self.sequence.value, TEST_SEQUENCE_INITIAL_VALUE)

        self.assertEqual(
            self.sequence.next_value(), TEST_SEQUENCE_INITIAL_VALUE
        )
        self.assertEqual(
            self.sequence.next_value(),
            TEST_SEQUENCE_INITIAL_VALUE + TEST_SEQUENCE_INCREMENT
        )

        self.assertEqual(
            self.sequence.value,
            TEST_SEQUENCE_INITIAL_VALUE + TEST_SEQUENCE_INCREMENT +
            TEST_SEQUENCE_INCREMENT
        )


@override_settings(OCR_AUTO_OCR=False)
class RenamingTemplateTestCase(TestCase):
    def setUp(self):
        self.sequence = Sequence.objects.create(
            label=TEST_SEQUENCE_LABEL, slug=TEST_SEQUENCE_SLUG,
            increment=TEST_SEQUENCE_INCREMENT,
            value=TEST_SEQUENCE_INITIAL_VALUE
        )
        self.document_type = DocumentType.objects.create(
            label=TEST_DOCUMENT_TYPE
        )
        self.document_type.renaming_template.template = TEST_TEMPLATE
        self.document_type.renaming_template.save()
        self.renaming_template = self.document_type.renaming_template

    def tearDown(self):
        self.document_type.delete()
        self.sequence.delete()

    def test_template_rendering(self):
        result = self.renaming_template.render_for(document=None)
        self.assertEqual(result, 'invoice-0')

        result = self.renaming_template.render_for(document=None)
        self.assertEqual(result, 'invoice-1')

    def test_rename_on_upload(self):
        with open(TEST_SMALL_DOCUMENT_PATH) as file_object:
            self.document = self.document_type.new_document(
                file_object=File(file_object)
            )

        document = Document.objects.get(pk=self.document.pk)
        self.assertEqual(document.label, 'invoice-0')



from __future__ import unicode_literals

import os

from documents.tests import TEST_SMALL_DOCUMENT_FILENAME

TEST_SMALL_DOCUMENT_PATH = os.path.join(
    'contrib', 'sample_documents',
    TEST_SMALL_DOCUMENT_FILENAME
)

TEST_SEQUENCE_LABEL = 'test sequence label'
TEST_SEQUENCE_LABEL_EDITED = 'test sequence label edited'
TEST_SEQUENCE_SLUG = 'test_sequence_slug'
TEST_SEQUENCE_INCREMENT = 1
TEST_SEQUENCE_INITIAL_VALUE = 0
TEST_TEMPLATE = 'invoice-{{{{ sequence_{}.next_value }}}}'.format(
    TEST_SEQUENCE_SLUG
)
TEST_TEMPLATE_EDITED = 'edited'



from __future__ import unicode_literals

DEFAULT_SEQUENCE_INCREMENT = 1
DEFAULT_SEQUENCE_INITIAL_VALUE = 0
DOCUMENT_RENAME_RETRY_DELAY = 10  # time in seconds

# time in seconds, should not be lower than DOCUMENT_RENAME_RETRY_DELAY
LOCK_EXPIRE = 30



from __future__ import unicode_literals

from django.contrib import admin

from .models import RenamingTemplate, Sequence


@admin.register(RenamingTemplate)
class RenamingTemplateAdmin(admin.ModelAdmin):
    list_display = ('document_type',)


@admin.register(Sequence)
class SequenceAdmin(admin.ModelAdmin):
    list_display = ('label', 'slug', 'increment', 'value')
    list_display_links = ('label', 'slug',)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('documents', '0028_newversionblock'),
    ]

    operations = [
        migrations.CreateModel(
            name='RenamingTemplate',
            fields=[
                (
                    'id', models.AutoField(
                        verbose_name='ID', serialize=False, auto_created=True,
                        primary_key=True
                    )
                ),
                (
                    'template', models.TextField(verbose_name='Template')
                ),
                (
                    'document_type', models.OneToOneField(
                        related_name='rename_template',
                        verbose_name='Document type',
                        to='documents.DocumentType'
                    )
                ),
            ],
            options={
                'verbose_name': 'Renaming template',
                'verbose_name_plural': 'Renaming templates',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Sequence',
            fields=[
                (
                    'id', models.AutoField(
                        verbose_name='ID', serialize=False, auto_created=True,
                        primary_key=True
                    )
                ),
                (
                    'label', models.CharField(
                        unique=True, max_length=32, verbose_name='Label'
                    )
                ),
                ('slug', models.SlugField(verbose_name='Slug')),
                ('value', models.IntegerField(verbose_name='Value')),
            ],
            options={
                'verbose_name': 'Sequence',
                'verbose_name_plural': 'Sequences',
            },
            bases=(models.Model,),
        ),
    ]



from __future__ import unicode_literals

from django.test import override_settings

from common.tests.test_views import GenericViewTestCase
from documents.models import DocumentType
from documents.permissions import permission_document_type_view
from documents.tests import TEST_DOCUMENT_TYPE
from user_management.tests.literals import (
    TEST_USER_PASSWORD, TEST_USER_USERNAME
)

from ..models import Sequence
from ..permissions import (
    permission_sequence_create, permission_sequence_delete,
    permission_sequence_edit, permission_sequence_view,
    permission_renaming_template_edit
)

from .literals import (
    TEST_SEQUENCE_LABEL, TEST_SEQUENCE_LABEL_EDITED, TEST_SEQUENCE_INCREMENT,
    TEST_SEQUENCE_INITIAL_VALUE, TEST_SEQUENCE_SLUG, TEST_TEMPLATE,
    TEST_TEMPLATE_EDITED
)


@override_settings(OCR_AUTO_OCR=False)
class SequenceViewsTestCase(GenericViewTestCase):
    def test_sequence_creation_view_no_permission(self):
        self.login(
            username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD
        )

        self.role.permissions.add(
            permission_sequence_view.stored_permission
        )
        response = self.post(
            'document_renaming:sequence_create',
            data={
                'label': TEST_SEQUENCE_LABEL,
                'slug': TEST_SEQUENCE_SLUG,
                'increment': TEST_SEQUENCE_INCREMENT,
                'value': TEST_SEQUENCE_INITIAL_VALUE,
            }, follow=True
        )
        self.assertEqual(response.status_code, 403)

    def test_sequence_creation_view_with_permission(self):
        self.login(
            username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD
        )

        self.role.permissions.add(
            permission_sequence_create.stored_permission
        )
        self.role.permissions.add(
            permission_sequence_view.stored_permission
        )
        response = self.post(
            'document_renaming:sequence_create',
            data={
                'label': TEST_SEQUENCE_LABEL,
                'slug': TEST_SEQUENCE_SLUG,
                'increment': TEST_SEQUENCE_INCREMENT,
                'value': TEST_SEQUENCE_INITIAL_VALUE,
            }, follow=True
        )
        self.assertEqual(response.status_code, 200)
        self.assertQuerysetEqual(
            Sequence.objects.all(),
            ('<Sequence: {}>'.format(TEST_SEQUENCE_LABEL),)
        )

    def test_sequence_delete_view_no_permission(self):
        self.login(
            username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD
        )
        sequence = Sequence.objects.create(
            slug=TEST_SEQUENCE_SLUG, label=TEST_SEQUENCE_LABEL
        )

        self.role.permissions.add(
            permission_sequence_view.stored_permission
        )
        response = self.post(
            'document_renaming:sequence_delete', args=(sequence.pk,),
            follow=True
        )
        self.assertEqual(response.status_code, 403)

    def test_sequence_delete_view_with_permission(self):
        self.login(
            username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD
        )
        sequence = Sequence.objects.create(
            slug=TEST_SEQUENCE_SLUG, label=TEST_SEQUENCE_LABEL
        )

        self.role.permissions.add(
            permission_sequence_delete.stored_permission
        )
        self.role.permissions.add(
            permission_sequence_view.stored_permission
        )
        response = self.post(
            'document_renaming:sequence_delete', args=(sequence.pk,),
            follow=True
        )
        self.assertEqual(response.status_code, 200)
        self.assertQuerysetEqual(
            Sequence.objects.all(), ()
        )

    def test_sequence_edit_view_no_permission(self):
        self.login(
            username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD
        )
        sequence = Sequence.objects.create(
            slug=TEST_SEQUENCE_SLUG, label=TEST_SEQUENCE_LABEL
        )

        self.role.permissions.add(
            permission_sequence_view.stored_permission
        )
        response = self.post(
            'document_renaming:sequence_edit', args=(sequence.pk,),
            data={
                'label': TEST_SEQUENCE_LABEL_EDITED,
                'slug': TEST_SEQUENCE_SLUG,
                'increment': TEST_SEQUENCE_INCREMENT,
                'value': TEST_SEQUENCE_INITIAL_VALUE
            }, follow=True
        )
        self.assertEqual(response.status_code, 403)
        self.assertEqual(sequence.label, TEST_SEQUENCE_LABEL)

    def test_sequence_edit_view_with_permission(self):
        self.login(
            username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD
        )
        sequence = Sequence.objects.create(
            slug=TEST_SEQUENCE_SLUG, label=TEST_SEQUENCE_LABEL
        )

        self.role.permissions.add(
            permission_sequence_edit.stored_permission
        )
        self.role.permissions.add(
            permission_sequence_view.stored_permission
        )
        response = self.post(
            'document_renaming:sequence_edit', args=(sequence.pk,),
            data={
                'label': TEST_SEQUENCE_LABEL_EDITED,
                'slug': TEST_SEQUENCE_SLUG,
                'increment': TEST_SEQUENCE_INCREMENT,
                'value': TEST_SEQUENCE_INITIAL_VALUE
            }, follow=True
        )
        self.assertEqual(response.status_code, 200)
        sequence = Sequence.objects.get(pk=sequence.pk)
        self.assertEqual(sequence.label, TEST_SEQUENCE_LABEL_EDITED)


@override_settings(OCR_AUTO_OCR=False)
class RenamingTemplateEditViewsTestCase(GenericViewTestCase):
    def setUp(self):
        super(RenamingTemplateEditViewsTestCase, self).setUp()
        self.document_type = DocumentType.objects.create(
            label=TEST_DOCUMENT_TYPE
        )

    def test_renaming_template_edit_view_no_permission(self):
        self.login(
            username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD
        )
        self.document_type.renaming_template.template = TEST_TEMPLATE
        self.document_type.renaming_template.save()

        self.role.permissions.add(
            permission_document_type_view.stored_permission
        )
        response = self.post(
            'document_renaming:renaming_template_edit',
            args=(self.document_type.pk,), data={
                'template': TEST_TEMPLATE_EDITED,
            }, follow=True
        )
        self.assertEqual(response.status_code, 403)
        self.assertEqual(
            self.document_type.renaming_template.template, TEST_TEMPLATE
        )

    def test_renaming_template_edit_view_with_permission(self):
        self.login(
            username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD
        )
        self.document_type.renaming_template.template = TEST_TEMPLATE
        self.document_type.renaming_template.save()

        self.role.permissions.add(
            permission_renaming_template_edit.stored_permission
        )
        self.role.permissions.add(
            permission_document_type_view.stored_permission
        )
        response = self.post(
            'document_renaming:renaming_template_edit',
            args=(self.document_type.pk,), data={
                'template': TEST_TEMPLATE_EDITED,
            }, follow=True
        )

        # Refresh the model
        self.document_type = DocumentType.objects.get(pk=self.document_type.pk)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            self.document_type.renaming_template.template, TEST_TEMPLATE_EDITED
        )



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from navigation import Link

from .permissions import (
    permission_sequence_create, permission_sequence_delete,
    permission_sequence_edit, permission_sequence_view,
    permission_renaming_template_edit
)

link_sequence_create = Link(
    permissions=(permission_sequence_create,),
    text=_('Create sequence'), view='document_renaming:sequence_create'
)
link_sequence_delete = Link(
    permissions=(permission_sequence_delete,), tags='dangerous',
    text=_('Delete'), view='document_renaming:sequence_delete',
    args='resolved_object.id'
)
link_sequence_edit = Link(
    permissions=(permission_sequence_edit,), text=_('Edit'),
    view='document_renaming:sequence_edit', args='resolved_object.id'
)
link_sequence_list = Link(
    icon='fa fa-sort-amount-asc', permissions=(permission_sequence_view,),
    text=_('Sequences'), view='document_renaming:sequence_list',
)

link_document_type_renaming_template_edit = Link(
    permissions=(permission_renaming_template_edit,),
    text=_('Renaming template'),
    view='document_renaming:renaming_template_edit', args='resolved_object.id'
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('document_renaming', '0004_auto_20151220_0751'),
    ]

    operations = [
        migrations.AlterField(
            model_name='renamingtemplate',
            name='template',
            field=models.TextField(
                help_text='Sequences are available as '
                '"sequence_[sequence slug]". To obtain the next value for a '
                'sequence, call it\'s next_value method, ie: '
                '"invoice-{{ sequence_invoices.next_value }}.',
                verbose_name='Template', blank=True
            ),
            preserve_default=True,
        ),
    ]



from __future__ import unicode_literals

import logging

from .models import RenamingTemplate
from .tasks import task_rename_document

logger = logging.getLogger(__name__)


def initialize_new_document_types(sender, instance, **kwargs):
    """
    Create a blank renaming template when a new document type is created
    """

    if kwargs['created']:
        RenamingTemplate.objects.create(document_type=instance)


def rename_document(sender, instance, **kwargs):
    """
    Call the background task to rename a document
    """

    logger.debug('instance: %s', instance)

    task_rename_document.apply_async(kwargs={'document_pk': instance.pk})



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from permissions import PermissionNamespace

namespace = PermissionNamespace('document_renaming', _('Document renaming'))

permission_sequence_create = namespace.add_permission(
    name='sequence_create', label=_('Create sequences')
)
permission_sequence_delete = namespace.add_permission(
    name='sequence_delete', label=_('Delete sequences')
)
permission_sequence_edit = namespace.add_permission(
    name='sequence_edit', label=_('Edit sequences')
)
permission_sequence_view = namespace.add_permission(
    name='sequence_view', label=_('View sequences')
)

permission_renaming_template_edit = namespace.add_permission(
    name='renaming_template_edit', label=_('Edit renaming templates')
)



from __future__ import unicode_literals

import logging

from django.db import models

logger = logging.getLogger(__name__)


class RenamingTemplateManager(models.Manager):
    def rename_document(self, document):
        result = document.document_type.renaming_template.render_for(
            document=document
        )
        document.label = result
        document.save()



from __future__ import unicode_literals

from django.db.models.signals import post_save
from django.utils.translation import ugettext_lazy as _

from common import menu_object, menu_secondary, menu_setup
from common.apps import MayanAppConfig
from documents.models import Document, DocumentType
from documents.signals import post_document_created
from mayan.celery import app
from navigation import SourceColumn

from .handlers import initialize_new_document_types, rename_document
from .links import (
    link_sequence_create, link_sequence_delete, link_sequence_edit,
    link_sequence_list, link_document_type_renaming_template_edit
)
from .models import Sequence


class DocumentRenamingApp(MayanAppConfig):
    name = 'document_renaming'
    test = True
    verbose_name = _('Document renaming')

    def ready(self):
        super(DocumentRenamingApp, self).ready()

        SourceColumn(
            source=Sequence, label=_('Label'),
            attribute='label'
        )

        SourceColumn(
            source=Sequence, label=_('Slug'),
            attribute='slug'
        )

        SourceColumn(
            source=Sequence, label=_('Increment'),
            attribute='increment'
        )

        SourceColumn(
            source=Sequence, label=_('Value'),
            attribute='value'
        )

        # Since we work on uploaded documents, we reuse the 'uploads' queue
        # defined by the 'documents' app.
        app.conf.CELERY_ROUTES.update(
            {
                'document_renaming.tasks.task_rename_document': {
                    'queue': 'uploads'
                },
            }
        )

        menu_object.bind_links(
            links=(link_sequence_delete, link_sequence_edit,),
            sources=(Sequence,)
        )

        menu_object.bind_links(
            links=(link_document_type_renaming_template_edit,),
            sources=(DocumentType,)
        )

        menu_secondary.bind_links(
            links=(link_sequence_list, link_sequence_create),
            sources=(
                Sequence, 'document_renaming:sequence_create',
                'document_renaming:sequence_list'
            )
        )

        menu_setup.bind_links(links=(link_sequence_list,))

        post_document_created.connect(
            rename_document, dispatch_uid='rename_document',
            sender=Document
        )

        post_save.connect(
            initialize_new_document_types,
            dispatch_uid='initialize_new_document_types', sender=DocumentType
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('document_renaming', '0003_auto_20151220_0717'),
    ]

    operations = [
        migrations.AlterField(
            model_name='renamingtemplate',
            name='document_type',
            field=models.OneToOneField(
                related_name='renaming_template', verbose_name='Document type',
                to='documents.DocumentType'
            ),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='renamingtemplate',
            name='template',
            field=models.TextField(
                help_text='Sequences are available as '
                '"sequence_[sequence slug]". To obtain the next value for a '
                'sequence, call it\'s next_value method, ie: '
                '"invoice-{{ sequence_invoices.next_value }}.',
                verbose_name='Template'
            ),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations


def initialize_renaming_templates(apps, schema_editor):
    RenamingTemplate = apps.get_model('document_renaming', 'RenamingTemplate')
    DocumentType = apps.get_model('documents', 'DocumentType')

    for document_type in DocumentType.objects.all():
        RenamingTemplate.objects.get_or_create(document_type=document_type)


def destroy_renaming_templates(apps, schema_editor):
    RenamingTemplate = apps.get_model('document_renaming', 'RenamingTemplate')
    RenamingTemplate.objects.all().delete()


class Migration(migrations.Migration):

    dependencies = [
        ('document_renaming', '0005_auto_20151221_0402'),
    ]

    operations = [
        migrations.RunPython(
            initialize_renaming_templates, destroy_renaming_templates
        ),
    ]



from __future__ import unicode_literals

import logging

from django.db import OperationalError

from mayan.celery import app

from documents.models import Document
from lock_manager import Lock, LockError

from .literals import DOCUMENT_RENAME_RETRY_DELAY, LOCK_EXPIRE
from .models import RenamingTemplate

logger = logging.getLogger(__name__)


@app.task(bind=True, default_retry_delay=DOCUMENT_RENAME_RETRY_DELAY, ignore_result=True)
def task_rename_document(self, document_pk):
    lock_id = 'task_rename_document-%d' % document_pk

    try:
        logger.debug('trying to acquire lock: %s', lock_id)
        # Acquire a lock to avoid a race condition when incrementing the
        # sequence
        lock = Lock.acquire_lock(name=lock_id, timeout=LOCK_EXPIRE)
        logger.debug('acquired lock: %s', lock_id)
        try:
            logger.info(
                'Starting document rename for document: %s', document_pk
            )
            document = Document.objects.get(pk=document_pk)
            RenamingTemplate.objects.rename_document(document=document)
        except OperationalError as exception:
            logger.warning(
                'Operational error during document rename for document: %d; '
                '%s. Retrying.', document_pk, exception
            )
            raise self.retry(exc=exception)
        finally:
            lock.release()
    except LockError:
        logger.debug('unable to obtain lock: %s' % lock_id)
